@@ -19,9 +19,9 @@
 
 """Provides the interface to the AT-SPI Registry."""
 
-__id__        = "$Id: atspi.py,v 1.34 2006/06/10 00:32:37 wwalker Exp $"
-__version__   = "$Revision: 1.34 $"
-__date__      = "$Date: 2006/06/10 00:32:37 $"
+__id__        = "$Id: atspi.py,v 1.38 2006/07/18 14:15:43 wwalker Exp $"
+__version__   = "$Revision: 1.38 $"
+__date__      = "$Date: 2006/07/18 14:15:43 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -141,12 +141,40 @@ class Registry:
         if not self.__dict__.has_key("desktop"):
             self.desktop = self.registry.getDesktop(0)
 
+    def __blockPreventor(self):
+        """[[[TODO: HACK to attempt to prevent deadlocks.  We call time.sleep
+        here as a means to sidestep the global interpreter lock (GIL).  Take
+        a look at the following URLs for more information:
+
+        http://mail.python.org/pipermail/python-list/2002-October/126632.html
+        http://twistedmatrix.com/pipermail/twisted-python/2005-July/011052.html
+        http://www.pyzine.com/Issue001/Section_Articles/article_ThreadingGlobalInterpreter.html"""
+
+        time.sleep(0.0001) # Attempt to sidestep GIL
+        return True
+
     def start(self):
         """Starts event notification with the AT-SPI Registry.  This method
         only returns after 'stop' has been called.
         """
         Accessible.init(self)
-        bonobo.main()
+
+        # We'll try our own main loop to help debug things.  Code borrowed
+        # "The Whole PyGtk FAQ": http://www.async.com.br/faq/pygtk/
+        #
+        if settings.useBonoboMain:
+            debug.println(debug.LEVEL_CONFIGURATION,
+                          "atspi.start: using bonobo.main")
+            gobject.idle_add(self.__blockPreventor)
+            bonobo.main()
+        else:
+            debug.println(debug.LEVEL_CONFIGURATION,
+                          "atspi.start: using our custom main loop")
+            self.running = True
+            context = gobject.MainLoop().get_context()
+            while self.running:
+                time.sleep(0.0001) # Attempt to sidestep GIL
+                dispatched = context.iteration(False)
 
     def stop(self):
         """Unregisters any event or keystroke listeners registered with
@@ -156,7 +184,10 @@ class Registry:
         Accessible.shutdown(self)
         for listener in (self.__listeners + self.__keystrokeListeners):
             listener.deregister()
-        bonobo.main_quit()
+        if settings.useBonoboMain:
+            bonobo.main_quit()
+        else:
+            self.running = False
 
     def registerEventListener(self, callback, eventType):
         """Registers the given eventType and callback with the Registry.
@@ -824,7 +855,7 @@ class Accessible:
 
         role = self.accessible.getRoleName()
 
-        # [[[WDW - HACK to coalesce menu items with children into
+        # [[[TODO: HACK to coalesce menu items with children into
         # menus.  The menu demo in gtk-demo does this, and one
         # might view that as an edge case.  But, in
         # gnome-terminal, "Terminal" -> "Set Character Encoding"
@@ -841,12 +872,23 @@ class Accessible:
             and (self.childCount > 0):
                 role = rolenames.ROLE_MENU
 
-        # [[[WDW - HACK - we sometimes get an object with an unknown
-        # role but it's role changes later on and we are not notified.
-        # An example of this is gnome-terminal.  So...to help with
-        # this, we will not cache the role if it is unknown.  See
-        # http://bugzilla.gnome.org/show_bug.cgi?id=344218 for more
-        # info.]]]
+        # [[[TODO: HACK because Java gives us radio button role and
+        # check box role for menu item objects, instead of giving us
+        # radio menu item role and check menu item role (see SwingSet
+        # menus).]]]
+        #
+        if (self.parent) and (self.parent.role == rolenames.ROLE_MENU):
+            if (role == rolenames.ROLE_RADIO_BUTTON):
+                role = rolenames.ROLE_RADIO_MENU_ITEM
+            elif (role == rolenames.ROLE_CHECK_BOX):
+                role = rolenames.ROLE_CHECK_MENU_ITEM
+
+        # [[[TODO: HACK because we sometimes get an object with an
+        # unknown role but it's role changes later on and we are not
+        # notified.  An example of this is gnome-terminal.  So...to
+        # help with this, we will not cache the role if it is unknown.
+        # See http://bugzilla.gnome.org/show_bug.cgi?id=344218 for
+        # more info.]]]
         #
         if role and settings.cacheValues \
             and (role != rolenames.ROLE_UNKNOWN):
