@@ -23,37 +23,14 @@ each region can potentially backed by an Accessible object.  Depending upon
 the Accessible object, the cursor routing keys can be used to perform
 operations on the Accessible object, such as invoking default actions or
 moving the text caret.
-
-The logical structure maintained by this module is a potentially a very large
-2 dimensional text area.  For each line, the text for each region is merely
-concatenated together, possibly creating a line that is longer than the
-physical display.  In addition, there may be any number of lines.  This module
-uses the physical display as a viewport into this large area, and provides
-support for the user to press BrlTTY command keys to scroll this viewport to
-view all the content.  [[[TODO: WDW - the goal is to support multi-line
-displays, but I've only really built in support for a single line display.
-Logged as bugzilla bug 319752.]]]
-
-As a means to keep things simple for now, regions can only exist on a single
-line.  That is, a region cannot cross line boundaries.
-
-After initialization, a typical use of this module would be as follows:
-
-    braille.clear()
-    line = braille.Line()
-    line.addRegion(braille.Region(...))
-    line.addRegion(braille.Component(...))
-    textRegion = braille.Text(...)
-    line.addRegion(textRegion)
-    braille.addLine(line)
-    braille.setFocus(textRegion)
-    braille.refresh()
-
-NOTE: for the most part, this module will happily do as requested if it isn't
-initialized.  The only impact will be that nothing will be displayed on the
-Braille display.
 """
 
+__id__        = "$Id: braille.py,v 1.56 2006/06/27 13:59:59 wwalker Exp $"
+__version__   = "$Revision: 1.56 $"
+__date__      = "$Date: 2006/06/27 13:59:59 $"
+__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__license__   = "LGPL"
+
 import atspi
 import brl
 try:
@@ -68,6 +45,7 @@ except:
 import debug
 import eventsynthesizer
 import settings
+import util
 
 from orca_i18n import _                          # for gettext support
 from rolenames import getShortBrailleForRoleName # localized role names
@@ -301,17 +279,29 @@ class Text(Region):
         """
 
         self.accessible = accessible
-        result = atspi.getTextLineAtCaret(self.accessible)
-        self.caretOffset = result[1]
-        self.lineOffset = result[2]
+        [string, self.caretOffset, self.lineOffset] = \
+                 util.getTextLineAtCaret(self.accessible)
+
+        # Sometimes, gnome-terminal will give us very odd values when
+        # the user is editing using 'vi' and has positioned the caret
+        # at the first character of the first line.  In this case, we
+        # end up getting a very large negative number for the line offset.
+        # So, we just assume the user is at the first character.
+        #
+        if self.lineOffset < 0:
+            self.caretOffset = 0
+            self.lineOffset = 0
+            [string, startOffset, endOffset] = \
+                self.accessible.text.getTextAtOffset(
+                    0,
+                    atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
+
         cursorOffset = self.caretOffset - self.lineOffset
 
         self.label = label
         if self.label:
-            string = self.label + " " + result[0]
+            string = self.label + " " + string
             cursorOffset += len(self.label) + 1
-        else:
-            string = result[0]
 
         Region.__init__(self, string, cursorOffset)
 
@@ -322,7 +312,7 @@ class Text(Region):
         True.  Otherwise, return False.
         """
 
-        result = atspi.getTextLineAtCaret(self.accessible)
+        result = util.getTextLineAtCaret(self.accessible)
         caretOffset = result[1]
         lineOffset = result[2]
         cursorOffset = caretOffset - lineOffset
