@@ -18,23 +18,28 @@
 # Boston, MA 02111-1307, USA.
 
 """Utilities for obtaining braille strings for objects.  In general,
-there probably should be a singleton instance of the BrailleUtil
+there probably should be a singleton instance of the BrailleGenerator
 class.  For those wishing to override the braille generators, however,
 one can create a new instance and replace/extend the braille
 generators as they see fit."""
 
+__id__        = "$Id: braillegenerator.py,v 1.52 2006/07/14 18:09:02 wwalker Exp $"
+__version__   = "$Revision: 1.52 $"
+__date__      = "$Date: 2006/07/14 18:09:02 $"
+__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__license__   = "LGPL"
+
 import math
 
 import atspi
 import braille
 import debug
-
 import orca
 import rolenames
 import settings
+import util
 
 from orca_i18n import _                     # for gettext support
-from rolenames import getBrailleForRoleName # localized role names
 
 class BrailleGenerator:
     """Takes accessible objects and produces a list of braille Regions
@@ -141,7 +146,7 @@ class BrailleGenerator:
         self.brailleGenerators[rolenames.ROLE_WINDOW]              = \
              self._getBrailleRegionsForWindow
 
-    def _getBrailleTextForAccelerator(self, obj):
+    def _getTextForAccelerator(self, obj):
         """Returns a string to be displayed that describes the keyboard
         accelerator (and possibly shortcut) for the given object.
 
@@ -151,25 +156,28 @@ class BrailleGenerator:
         Returns a string to be displayed.
         """
 
-        text = ""
+        if settings.brailleVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE:
+            text = ""
 
-        result = atspi.getAcceleratorAndShortcut(obj)
+            result = util.getAcceleratorAndShortcut(obj)
 
-        accelerator = result[0]
-        #shortcut = result[1]
+            accelerator = result[0]
+            #shortcut = result[1]
 
-        if len(accelerator) > 0:
-            text += "(" + accelerator + ")"
+            if len(accelerator) > 0:
+                text += "(" + accelerator + ")"
 
-        # [[[TODO: WDW - various stuff preserved while we work out the
-        # desired verbosity here.]]]
-        #
-        #if len(shortcut) > 0:
-        #    text += "(" + shortcut + ")"
+            # [[[TODO: WDW - various stuff preserved while we work out the
+            # desired verbosity here.]]]
+            #
+            #if len(shortcut) > 0:
+            #    text += "(" + shortcut + ")"
 
-        return text
+            return text
+        else:
+            return None
 
-    def _getBrailleTextForAvailability(self, obj):
+    def _getTextForAvailability(self, obj):
         """Returns a string to be displayed that describes the availability
         of the given object.
 
@@ -179,12 +187,14 @@ class BrailleGenerator:
         Returns a string to be displayed.
         """
 
-        if obj.state.count(atspi.Accessibility.STATE_SENSITIVE):
-            return ""
-        else:
+        if (settings.brailleVerbosityLevel \
+            == settings.VERBOSITY_LEVEL_VERBOSE) \
+            and obj.state.count(atspi.Accessibility.STATE_SENSITIVE) == 0:
             return _("grayed")
+        else:
+            return None
 
-    def _getBrailleTextForValue(self, obj):
+    def _getTextForValue(self, obj):
         """Returns the text to be displayed for the object's current value.
 
         Arguments:
@@ -232,6 +242,14 @@ class BrailleGenerator:
         #
         return valueString
 
+    def _getTextForRole(self, obj):
+        if (settings.brailleVerbosityLevel \
+            == settings.VERBOSITY_LEVEL_VERBOSE)\
+           and (obj.role != rolenames.ROLE_UNKNOWN):
+            return rolenames.getBrailleForRoleName(obj)
+        else:
+            return None
+
     def _debugGenerator(self, generatorName, obj):
         """Prints debug.LEVEL_FINER information regarding the braille
         generator.
@@ -264,21 +282,13 @@ class BrailleGenerator:
 
         self._debugGenerator("_getDefaultBrailleRegions", obj)
 
-        verbosity = settings.brailleVerbosityLevel
+        regions = []
 
-        text = obj.label
-
-        value = self._getBrailleTextForValue(obj)
-        if len(value) > 0:
-            if len(text):
-                text += " "
-            text += value
-
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            if (obj.label == None) or (obj.role != rolenames.ROLE_UNKNOWN):
-                if len(text):
-                    text += " "
-                text += getBrailleForRoleName(obj)
+        text = ""
+        text = util.appendString(text, util.getDisplayedLabel(obj))
+        text = util.appendString(text, util.getDisplayedText(obj))
+        text = util.appendString(text, self._getTextForValue(obj))
+        text = util.appendString(text, self._getTextForRole(obj))
 
         regions = []
         componentRegion = braille.Component(obj, text)
@@ -313,19 +323,11 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForAnimation", obj)
 
-        verbosity = settings.brailleVerbosityLevel
-
-        text = obj.label
-
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            if len(text):
-                text += " "
-            text += getBrailleForRoleName(obj)
-
-        if obj.description:
-            if len(text):
-                text += ": "
-            text += obj.description
+        text = ""
+        text = util.appendString(text, util.getDisplayedLabel(obj))
+        text = util.appendString(text, util.getDisplayedText(obj))
+        text = util.appendString(text, self._getTextForRole(obj))
+        text = util.appendString(text, obj.description, ": ")
 
         regions = []
         componentRegion = braille.Component(obj, text)
@@ -364,19 +366,16 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForCheckBox", obj)
 
-        verbosity = settings.brailleVerbosityLevel
-
-        text = obj.label
-        if len(text):
-            text += " "
+        text = ""
+        text = util.appendString(text, util.getDisplayedLabel(obj))
+        text = util.appendString(text, util.getDisplayedText(obj))
 
         if obj.state.count(atspi.Accessibility.STATE_CHECKED):
-            text += "<x>"
+            text = util.appendString(text, "<x>")
         else:
-            text += "< >"
+            text = util.appendString(text, "< >")
 
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            text += " " + getBrailleForRoleName(obj)
+        text = util.appendString(text, self._getTextForRole(obj))
 
         regions = []
         componentRegion = braille.Component(obj, text)
@@ -397,26 +396,21 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForCheckMenuItem", obj)
 
-        verbosity = settings.brailleVerbosityLevel
-
-        text = obj.label
-        if len(text):
-            text += " "
+        text = ""
+        text = util.appendString(text, util.getDisplayedLabel(obj))
+        text = util.appendString(text, util.getDisplayedText(obj))
 
         if obj.state.count(atspi.Accessibility.STATE_CHECKED):
-            text += "<x>"
+            text = util.appendString(text, "<x>")
         else:
-            text += "< >"
+            text = util.appendString(text, "< >")
 
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            if obj == orca.locusOfFocus:
-                text += " " + getBrailleForRoleName(obj)
-                availability = self._getBrailleTextForAvailability(obj)
-                if len(availability) > 0:
-                    text += " " + availability + " "
-                accelerator = self._getBrailleTextForAccelerator(obj)
-                if len(accelerator) > 0:
-                    text += accelerator
+        if obj == orca.locusOfFocus:
+            text = util.appendString(text, self._getTextForRole(obj))
+            text = util.appendString(text, self._getTextForAvailability(obj))
+            text = util.appendString(text,
+                                      self._getTextForAccelerator(obj),
+                                      "")
 
         regions = []
         componentRegion = braille.Component(obj, text)
@@ -451,67 +445,26 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForComboBox", obj)
 
-        verbosity = settings.brailleVerbosityLevel
-
         regions = []
-        regions.append(braille.Region(obj.label + " "))
 
-        # Find the text displayed in the combo box.  This is either:
-        #
-        # 1) The last text object that's a child of the combo box
-        # 2) The selected child of the combo box.
-        # 3) The contents of the text of the combo box itself when
-        #    treated as a text object.
-        #
-        # Preference is given to #1, if it exists.
-        #
-        # If the label of the combo box is the same as the utterance for
-        # the child object, then this utterance is only displayed once.
-        #
-        # [[[TODO: WDW - Combo boxes are complex beasts.  This algorithm
-        # needs serious work.  Logged as bugzilla bug 319745.]]]
-        #
-        textObj = None
-        for i in range(0, obj.childCount):
-            debug.println(debug.LEVEL_FINEST,
-                          "braillegenerator._getBrailleRegionsForComboBox " \
-                          + "looking at child %d" % i)
-            child = obj.child(i)
-            if child.role == rolenames.ROLE_TEXT:
-                textObj = child
+        focusedRegionIndex = 0
+        label = util.getDisplayedLabel(obj)
+        if label and (len(label) > 0):
+            regions.append(braille.Region(label + " "))
+            focusedRegionIndex = 1
 
-        if textObj:
-            result = atspi.getTextLineAtCaret(textObj)
-            line = result[0]
-            if line != obj.label:
-                regions.append(braille.Text(textObj))
-        else:
-            selectedItem = None
-            comboSelection = obj.selection
-            if comboSelection and comboSelection.nSelectedChildren > 0:
-                selectedItem = atspi.Accessible.makeAccessible(\
-                    comboSelection.getSelectedChild(0))
-            if selectedItem:
-                if selectedItem.label != obj.label:
-                    regions.append(braille.Region(selectedItem.label))
-            else:
-                if obj.text:
-                    result = atspi.getTextLineAtCaret(obj)
-                    selectedText = result[0]
-                    if selectedText != obj.label:
-                        regions.append(braille.Text(obj))
-                else:
-                    debug.println(
-                        debug.LEVEL_SEVERE,
-                        "ERROR: Could not find selected item for combo box.")
+        displayedText = util.getDisplayedText(obj)
+        if displayedText:
+            regions.append(braille.Region(displayedText))
 
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            regions.append(braille.Region(" " + getBrailleForRoleName(obj)))
+        if settings.brailleVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE:
+            regions.append(braille.Region(
+                " " + rolenames.getBrailleForRoleName(obj)))
 
         # [[[TODO: WDW - perhaps if a text area was created, we should
         # give focus to it.]]]
         #
-        return [regions, regions[0]]
+        return [regions, regions[focusedRegionIndex]]
 
     def _getBrailleRegionsForDesktopIcon(self, obj):
         """Get the braille for a desktop icon.
@@ -612,21 +565,16 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForIcon", obj)
 
-        verbosity = settings.brailleVerbosityLevel
-
-        text = obj.label
+        text = ""
+        text = util.appendString(text, util.getDisplayedLabel(obj))
+        text = util.appendString(text, util.getDisplayedText(obj))
 
         if obj.image:
             description = obj.image.imageDescription
             if len(description):
-                if len(text):
-                    text += " "
-                text += description
+                text = util.appendString(text, description)
 
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            if len(text):
-                text += " "
-            text += getBrailleForRoleName(obj)
+        text = util.appendString(text, self._getTextForRole(obj))
 
         regions = []
         componentRegion = braille.Component(obj, text)
@@ -692,22 +640,16 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForMenu", obj)
 
-        verbosity = settings.brailleVerbosityLevel
+        text = ""
+        text = util.appendString(text, util.getDisplayedLabel(obj))
+        text = util.appendString(text, util.getDisplayedText(obj))
+        text = util.appendString(text, rolenames.getBrailleForRoleName(obj))
 
-        text = obj.label
-
-        if len(text):
-            text += " "
-        text += getBrailleForRoleName(obj)
-
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            if obj == orca.locusOfFocus:
-                availability = self._getBrailleTextForAvailability(obj)
-                if len(availability) > 0:
-                    text += " " + availability + " "
-                accelerator = self._getBrailleTextForAccelerator(obj)
-                if len(accelerator) > 0:
-                    text += accelerator
+        if obj == orca.locusOfFocus:
+            text = util.appendString(text, self._getTextForAvailability(obj))
+            text = util.appendString(text,
+                                      self._getTextForAccelerator(obj),
+                                      "")
 
         regions = []
         componentRegion = braille.Component(obj, text)
@@ -741,22 +683,15 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForMenuItem", obj)
 
-        verbosity = settings.brailleVerbosityLevel
+        text = ""
+        text = util.appendString(text, util.getDisplayedLabel(obj))
+        text = util.appendString(text, util.getDisplayedText(obj))
 
-        text = obj.label
-
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            if obj == orca.locusOfFocus:
-                # [[[TODO - WDW comment this out for now until we work
-                # out verbosity.]]]
-                #
-                #text += " " + getBrailleForRoleName(obj)
-                availability = self._getBrailleTextForAvailability(obj)
-                if len(availability) > 0:
-                    text += " " + availability + " "
-                accelerator = self._getBrailleTextForAccelerator(obj)
-                if len(accelerator) > 0:
-                    text += accelerator
+        if obj == orca.locusOfFocus:
+            text = util.appendString(text, self._getTextForAvailability(obj))
+            text = util.appendString(text,
+                                      self._getTextForAccelerator(obj),
+                                      "")
 
         regions = []
         componentRegion = braille.Component(obj, text)
@@ -778,17 +713,13 @@ class BrailleGenerator:
 
         regions = []
 
-        textRegion = braille.Text(obj, obj.label)
+        textRegion = braille.Text(obj, util.getDisplayedLabel(obj))
         regions.append(textRegion)
 
         eol = braille.Region(" $l")
         regions.append(eol)
 
         # We do not want the role at the end of text areas.
-        #
-        #if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-        #    text = " " + getBrailleForRoleName(obj)
-        #regions.append(braille.Region(text))
 
         return [regions, textRegion]
 
@@ -818,16 +749,15 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForPageTab", obj)
 
-        verbosity = settings.brailleVerbosityLevel
+        text = ""
+        text = util.appendString(text, util.getDisplayedLabel(obj))
+        text = util.appendString(text, util.getDisplayedText(obj))
 
-        text = obj.label
-
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            if obj == orca.locusOfFocus:
-                text += " " + getBrailleForRoleName(obj)
-                accelerator = self._getBrailleTextForAccelerator(obj)
-                if len(accelerator) > 0:
-                    text += accelerator
+        if obj == orca.locusOfFocus:
+            text = util.appendString(text, self._getTextForAvailability(obj))
+            text = util.appendString(text,
+                                      self._getTextForAccelerator(obj),
+                                      "")
 
         regions = []
         componentRegion = braille.Component(obj, text)
@@ -876,7 +806,25 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForPushButton", obj)
 
-        return self._getDefaultBrailleRegions(obj)
+        regions = []
+
+        text = ""
+        text = util.appendString(text, util.getDisplayedLabel(obj))
+        text = util.appendString(text, util.getDisplayedText(obj))
+        text = util.appendString(text, self._getTextForRole(obj))
+
+        # In Java, some push buttons don't have label and text.
+        # In this case, we'll add to presentation the object description,
+        # if exists.
+        #
+        if (not text) and (obj.description):
+            text = util.appendString(text, obj.description)
+
+        regions = []
+        componentRegion = braille.Component(obj, text)
+        regions.append(componentRegion)
+
+        return [regions, componentRegion]
 
     def _getBrailleRegionsForRadioButton(self, obj):
         """Get the braille for a radio button.  If the button already had
@@ -891,19 +839,23 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForRadioButton", obj)
 
-        verbosity = settings.brailleVerbosityLevel
+        text = ""
+        text = util.appendString(text, util.getDisplayedLabel(obj))
+        text = util.appendString(text, util.getDisplayedText(obj))
 
-        text = obj.label
-        if len(text):
-            text += " "
+        # In Java, some toggle buttons don't have label and text.
+        # In this case, we'll add to presentation the object description,
+        # if exists.
+        #
+        if (not text) and (obj.description):
+            text = util.appendString(text, obj.description)
 
         if obj.state.count(atspi.Accessibility.STATE_CHECKED):
-            text += "&=y"
+            text = util.appendString(text, "&=y")
         else:
-            text += "& y"
+            text = util.appendString(text, "& y")
 
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            text += " " + getBrailleForRoleName(obj)
+        text = util.appendString(text, self._getTextForRole(obj))
 
         regions = []
         componentRegion = braille.Component(obj, text)
@@ -924,26 +876,21 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForRadioMenuItem", obj)
 
-        verbosity = settings.brailleVerbosityLevel
-
-        text = obj.label
-        if len(text):
-            text += " "
+        text = ""
+        text = util.appendString(text, util.getDisplayedLabel(obj))
+        text = util.appendString(text, util.getDisplayedText(obj))
 
         if obj.state.count(atspi.Accessibility.STATE_CHECKED):
-            text += "&=y"
+            text = util.appendString(text, "&=y")
         else:
-            text += "& y"
+            text = util.appendString(text, "& y")
 
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            if obj == orca.locusOfFocus:
-                text += " " + getBrailleForRoleName(obj)
-                availability = self._getBrailleTextForAvailability(obj)
-                if len(availability) > 0:
-                    text += " " + availability + " "
-                accelerator = self._getBrailleTextForAccelerator(obj)
-                if len(accelerator) > 0:
-                    text += accelerator
+        if obj == orca.locusOfFocus:
+            text = util.appendString(text, self._getTextForRole(obj))
+            text = util.appendString(text, self._getTextForAvailability(obj))
+            text = util.appendString(text,
+                                      self._getTextForAccelerator(obj),
+                                      "")
 
         regions = []
         componentRegion = braille.Component(obj, text)
@@ -993,9 +940,22 @@ class BrailleGenerator:
         and the second element is the Region which should get focus.
         """
 
-        self._debugGenerator("_getBrailleRegionsForProgressBar", obj)
+        self._debugGenerator("_getBrailleRegionsForSlider", obj)
 
-        return self._getDefaultBrailleRegions(obj)
+        regions = []
+
+        text = ""
+        text = util.appendString(text, util.getDisplayedLabel(obj))
+        # Ignore the text on the slider.
+        #text = util.appendString(text, util.getDisplayedText(obj))
+        text = util.appendString(text, self._getTextForValue(obj))
+        text = util.appendString(text, self._getTextForRole(obj))
+
+        regions = []
+        componentRegion = braille.Component(obj, text)
+        regions.append(componentRegion)
+
+        return [regions, componentRegion]
 
     def _getBrailleRegionsForSpinButton(self, obj):
         """Get the braille for a spin button.  If the object already has
@@ -1070,12 +1030,51 @@ class BrailleGenerator:
                     regions = self._getBrailleRegionsForCheckBox(obj)
                     obj.role = rolenames.ROLE_TABLE_CELL
                     break
-                #elif action.getName(i) == "edit":
-                #    text = self._getSpeechForText(obj, True)
-                #    break
 
         if len(regions) == 0:
-            regions = self._getDefaultBrailleRegions(obj)
+            if settings.readTableCellRow:
+                rowRegions = []
+                savedBrailleVerbosityLevel = settings.brailleVerbosityLevel
+                settings.brailleVerbosityLevel = \
+                                             settings.VERBOSITY_LEVEL_BRIEF
+
+                parent = obj.parent
+                row = parent.table.getRowAtIndex(obj.index)
+                column = parent.table.getColumnAtIndex(obj.index)
+
+                # This is an indication of whether we should speak all the
+                # table cells (the user has moved focus up or down a row),
+                # or just the current one (focus has moved left or right in
+                # the same row).
+                #
+                speakAll = True
+                if parent.__dict__.has_key("lastRow") and \
+                    parent.__dict__.has_key("lastColumn"):
+                    speakAll = (parent.lastRow != row) or \
+                           ((row == 0 or row == parent.table.nRows-1) and \
+                            parent.lastColumn == column)
+
+                if speakAll == True:
+                    focusRowRegion = None
+                    for i in range(0, parent.table.nColumns):
+                        accRow = parent.table.getAccessibleAt(row, i)
+                        cell = atspi.Accessible.makeAccessible(accRow)
+                        [cellRegions, focusRegion] = \
+                            self._getDefaultBrailleRegions(\
+                                util.getRealActiveDescendant(cell))
+                        if len(rowRegions):
+                            rowRegions.append(braille.Region(" "))
+                        rowRegions.append(cellRegions[0])
+                        if i == column:
+                            focusRowRegion = cellRegions[0]
+                    regions = [rowRegions, focusRowRegion]
+                    settings.brailleVerbosityLevel = savedBrailleVerbosityLevel
+                else:
+                    regions = self._getDefaultBrailleRegions(
+                        util.getRealActiveDescendant(obj))
+            else:
+                regions = self._getDefaultBrailleRegions(
+                    util.getRealActiveDescendant(obj))
 
         # [[[TODO: WDW - HACK attempt to determine if this is a node;
         # if so, describe its state.]]]
@@ -1086,7 +1085,7 @@ class BrailleGenerator:
             else:
                 regions[0].append(braille.Region(" " + _("collapsed")))
 
-        level = atspi.getNodeLevel(obj)
+        level = util.getNodeLevel(obj)
         if level >= 0:
             regions[0].append(braille.Region(" " + _("TREE LEVEL %d") \
                                              % (level + 1)))
@@ -1133,13 +1132,10 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForTearOffMenuItem", obj)
 
-        text = getBrailleForRoleName(obj)
-
-        regions = []
-        componentRegion = braille.Component(obj, text)
-        regions.append(componentRegion)
-
-        return [regions, componentRegion]
+        componentRegion = braille.Component(
+            obj,
+            rolenames.getBrailleForRoleName(obj))
+        return [[componentRegion], componentRegion]
 
     def _getBrailleRegionsForTerminal(self, obj):
         """Get the braille for a terminal
@@ -1153,20 +1149,15 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForTerminal", obj)
 
-        verbosity = settings.brailleVerbosityLevel
-
-        label = None
-        frame = atspi.getFrame(obj)
+        title = None
+        frame = util.getFrame(obj)
         if frame:
-            label = frame.name
-        if not label:
-            label = obj.label
-        text = label
+            title = frame.name
+        if not title:
+            title = util.getDisplayedLabel(obj)
 
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            if len(text):
-                text += " "
-            text += getBrailleForRoleName(obj)
+        text = title
+        text = util.appendString(text, rolenames.getBrailleForRoleName(obj))
 
         regions = []
         regions.append(braille.Region(text))
@@ -1258,11 +1249,14 @@ class BrailleGenerator:
         - groupChildren: if True, children of an object should be displayed
                          together with their parent, where each child is
                          separated by _ and the selected child is the Region
-                         that should get focus.
+                         that should get focus.  The default here is True,
+                         but this also is used in conjunction with
+                         settings.enableBrailleGrouping.
 
         Returns a list where the first element is a list of Regions to
         display and the second element is the Region which should get
-        focus.  """
+        focus.
+        """
 
         # If we want to group the children, first see if obj is a child of
         # something we like to group.  If so, then reset the obj to the obj's
@@ -1270,7 +1264,7 @@ class BrailleGenerator:
         # to group.  If all fails, we don't try grouping.
         #
         reallyGroupChildren = False
-        if groupChildren:
+        if settings.enableBrailleGrouping and groupChildren:
             parent = obj.parent
             isChild = parent \
                       and ((parent.role == rolenames.ROLE_MENU) \
@@ -1307,7 +1301,7 @@ class BrailleGenerator:
                 # Sometimes, for some unknown reason, the child is None.
                 # We now test for this, rather than cause a traceback.
                 #
-                if child and child.role != rolenames.ROLE_SEPARATOR:
+                if child and (child.role != rolenames.ROLE_SEPARATOR):
 
                 # the following line has been removed because insensitive
                 # menu items can get focus in StarOffice.
@@ -1343,8 +1337,6 @@ class BrailleGenerator:
         Returns a list of Regions to display.
         """
 
-        verbosity = settings.brailleVerbosityLevel
-
         brailleRolenameStyle = settings.brailleRolenameStyle
 
         regions = []
@@ -1362,12 +1354,32 @@ class BrailleGenerator:
             #
             if (parent.role != rolenames.ROLE_FILLER) \
                 and (parent.role != rolenames.ROLE_SPLIT_PANE):
-                if (len(parent.label) > 0) \
+
+                # Announce the label and text of the object in the hierarchy.
+                #
+                label = util.getDisplayedLabel(parent)
+                text = util.getDisplayedText(parent)
+                if ((label and len(label)) or (text and len(text))) \
                     or (parent.role != rolenames.ROLE_PANEL):
                     regions.append(braille.Region(" "))
                     result = self.getBrailleRegions(parent, False)
                     regions.extend(result[0])
 
+            # [[[TODO: HACK - we've discovered oddness in hierarchies
+            # such as the gedit Edit->Preferences dialog.  In this
+            # dialog, we have labeled groupings of objects.  The
+            # grouping is done via a FILLER with two children - one
+            # child is the overall label, and the other is the
+            # container for the grouped objects.  When we detect this,
+            # we add the label to the overall context.]]]
+            #
+            if parent.role == rolenames.ROLE_FILLER:
+                label = util.getDisplayedLabel(parent)
+                if label and len(label) and not label.isspace():
+                    regions.append(braille.Region(" "))
+                    result = self.getBrailleRegions(parent, False)
+                    regions.extend(result[0])
+
             parent = parent.parent
 
         regions.reverse()
@@ -1383,14 +1395,15 @@ class BrailleGenerator:
             row = parent.table.getRowAtIndex(obj.index)
             desc = parent.table.getRowDescription(row)
             if desc and len(desc):
-                if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
+                if settings.brailleRolenameStyle \
+                       == settings.VERBOSITY_LEVEL_VERBOSE:
                     if brailleRolenameStyle \
                            == settings.BRAILLE_ROLENAME_STYLE_LONG:
                         text = desc + " " + rolenames.rolenames[\
-                            rolenames.ROLE_ROW_HEADER].brailleLong
+                            rolenames.ROLE_ROW_HEADER].brailleLong + " "
                     else:
                         text = desc + " " + rolenames.rolenames[\
-                            rolenames.ROLE_ROW_HEADER].brailleShort
+                            rolenames.ROLE_ROW_HEADER].brailleShort + " "
                 else:
                     text = desc
                 regions.append(braille.Region(text))
@@ -1398,14 +1411,15 @@ class BrailleGenerator:
             col = parent.table.getColumnAtIndex(obj.index)
             desc = parent.table.getColumnDescription(col)
             if desc and len(desc):
-                if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
+                if settings.brailleVerbosityLevel \
+                       == settings.VERBOSITY_LEVEL_VERBOSE:
                     if brailleRolenameStyle \
                            == settings.BRAILLE_ROLENAME_STYLE_LONG:
                         text = desc + " " + rolenames.rolenames[\
-                            rolenames.ROLE_COLUMN_HEADER].brailleLong
+                            rolenames.ROLE_COLUMN_HEADER].brailleLong + " "
                     else:
                         text = desc + " " + rolenames.rolenames[\
-                            rolenames.ROLE_COLUMN_HEADER].brailleShort
+                            rolenames.ROLE_COLUMN_HEADER].brailleShort + " "
                 else:
                     text = desc
                 regions.append(braille.Region(text))
