@@ -19,9 +19,9 @@
 
 """The main module for the Orca screen reader."""
 
-__id__        = "$Id: orca.py,v 1.141 2006/06/29 16:13:15 richb Exp $"
-__version__   = "$Revision: 1.141 $"
-__date__      = "$Date: 2006/06/29 16:13:15 $"
+__id__        = "$Id: orca.py,v 1.144 2006/07/19 20:46:28 richb Exp $"
+__version__   = "$Revision: 1.144 $"
+__date__      = "$Date: 2006/07/19 20:46:28 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -80,6 +80,10 @@ lastInputEvent = None
 #
 lastTimestamp = None
 
+# The time that the last "No focus" event occured.
+#
+noFocusTimeStamp = 0.0
+
 # A new modifier to use (set by the press of any key in the
 # settings.orcaModifierKeys list) to represent the Orca modifier.
 #
@@ -260,7 +264,7 @@ def _onMouseButton(e):
     #
     if event.type.endswith("p"):
         speech.stop()
-    
+
 ########################################################################
 #                                                                      #
 # Keyboard Event Recording Support                                     #
@@ -613,6 +617,8 @@ def _keyEcho(event):
     voices = settings.voices
     voice = voices[settings.DEFAULT_VOICE]
 
+    state = None
+
     # If key echo is enabled, then check to see what type of key event
     # it is and echo it via speech, if the user wants that type of key
     # echoed.
@@ -637,9 +643,10 @@ def _keyEcho(event):
             modifiers = event.modifiers
             if event_string == _("Caps_Lock"):
                 if modifiers & (1 << atspi.Accessibility.MODIFIER_SHIFTLOCK):
-                    event_string += " Off"
+                    state = _(" off")
                 else:
-                    event_string += " On"
+                    state = _(" on")
+
             elif event_string == _("Num_Lock"):
                 # [[[TODO: richb - we are not getting a correct modifier
                 # state value returned when Num Lock is turned off.
@@ -647,9 +654,9 @@ def _keyEcho(event):
                 # until this can be fixed.]]]
                 #
                 #if modifiers & (1 << atspi.Accessibility.MODIFIER_NUMLOCK):
-                #    event_string += " Off"
+                #    event_string += _(" off")
                 #else:
-                #    event_string += " On"
+                #    event_string += _(" on")
                 pass
 
         elif _isFunctionKey(event_string):
@@ -670,6 +677,9 @@ def _keyEcho(event):
         if event_string in keynames.keynames:
             event_string = keynames.keynames[event_string]
 
+        if state:
+            event_string += state
+
         debug.println(debug.LEVEL_FINEST,
                       "orca._keyEcho: speaking: %s" % event_string)
 
@@ -900,7 +910,7 @@ def loadUserSettings(script=None, inputEvent=None):
                       "Magnification module has NOT been initialized.")
 
     httpserver.init()
-    
+
     return True
 
 def _showPreferencesGUI(script=None, inputEvent=None):
@@ -960,140 +970,12 @@ def init(registry):
     if _initialized:
         return False
 
+    # Note that we have moved the Orca specific keybindings to the default
+    # script, so _keyBindings is currently empty. The logic is retained
+    # here, just in case we wish to reinstate them in the future.
+    #
     _keyBindings = keybindings.KeyBindings()
 
-    enterLearnModeHandler = InputEventHandler(\
-        enterLearnMode,
-        _("Enters learn mode.  Press escape to exit learn mode."))
-    _keyBindings.add(keybindings.KeyBinding("F1", \
-                                            1 << MODIFIER_ORCA, \
-                                            1 << MODIFIER_ORCA, \
-                                            enterLearnModeHandler))
-
-    decreaseSpeechRateHandler = InputEventHandler(\
-        speech.decreaseSpeechRate,
-        _("Decreases the speech rate."))
-    _keyBindings.add(keybindings.KeyBinding("Left", \
-                                            1 << MODIFIER_ORCA, \
-                                            1 << MODIFIER_ORCA,
-                                            decreaseSpeechRateHandler))
-
-    increaseSpeechRateHandler = InputEventHandler(\
-        speech.increaseSpeechRate,
-        _("Increases the speech rate."))
-    _keyBindings.add(keybindings.KeyBinding("Right", \
-                                            1 << MODIFIER_ORCA, \
-                                            1 << MODIFIER_ORCA,
-                                            increaseSpeechRateHandler))
-
-    decreaseSpeechPitchHandler = InputEventHandler(\
-        speech.decreaseSpeechPitch,
-        _("Decreases the speech pitch."))
-    _keyBindings.add(keybindings.KeyBinding("Down", \
-                                            1 << MODIFIER_ORCA, \
-                                            1 << MODIFIER_ORCA,
-                                            decreaseSpeechPitchHandler))
-
-    increaseSpeechPitchHandler = InputEventHandler(\
-        speech.increaseSpeechPitch,
-        _("Increases the speech pitch."))
-    _keyBindings.add(keybindings.KeyBinding("Up", \
-                                            1 << MODIFIER_ORCA, \
-                                            1 << MODIFIER_ORCA,
-                                            increaseSpeechPitchHandler))
-
-    shutdownHandler = InputEventHandler(shutdown, _("Quits Orca"))
-    _keyBindings.add(keybindings.KeyBinding("q", \
-                                            1 << MODIFIER_ORCA, \
-                                            1 << MODIFIER_ORCA,
-                                            shutdownHandler))
-
-    keystrokeRecordingHandler = InputEventHandler(\
-        toggleKeystrokeRecording,
-        _("Toggles keystroke recording on and off."))
-    _keyBindings.add(keybindings.KeyBinding("Pause", \
-                                            0, \
-                                            0,
-                                            keystrokeRecordingHandler))
-
-    preferencesSettingsHandler = InputEventHandler(\
-        _showPreferencesGUI,
-        _("Displays the preferences configuration dialog."))
-    _keyBindings.add(keybindings.KeyBinding(
-        "space", \
-        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL),
-        1 << MODIFIER_ORCA,
-        preferencesSettingsHandler))
-
-    loadUserSettingsHandler = InputEventHandler(\
-        loadUserSettings,
-        _("Reloads user settings and reinitializes services as necessary."))
-    _keyBindings.add(keybindings.KeyBinding(
-        "space", \
-        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL),
-        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL),
-        loadUserSettingsHandler))
-
-    toggleSilenceSpeechHandler = InputEventHandler(\
-        _toggleSilenceSpeech,
-        _("Toggles the silencing of speech."))
-    _keyBindings.add(keybindings.KeyBinding(
-        "s", \
-        1 << MODIFIER_ORCA,
-        1 << MODIFIER_ORCA,
-        toggleSilenceSpeechHandler))
-
-    listAppsHandler = InputEventHandler(
-        printApps,
-        _("Prints a debug listing of all known applications to the console where Orca is running."))
-    _keyBindings.add(keybindings.KeyBinding(
-        "F5",
-        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL),
-        1 << MODIFIER_ORCA,
-        listAppsHandler))
-
-    cycleDebugLevelHandler = InputEventHandler(
-        cycleDebugLevel,
-        _("Cycles the debug level at run time."))
-    _keyBindings.add(keybindings.KeyBinding("F4", \
-                                            1 << MODIFIER_ORCA,
-                                            1 << MODIFIER_ORCA,
-                                            cycleDebugLevelHandler))
-
-    printActiveAppHandler = InputEventHandler(\
-        printActiveApp,
-        _("Prints debug information about the currently active application to the console where Orca is running."))
-    _keyBindings.add(keybindings.KeyBinding("F6", \
-                                            1 << MODIFIER_ORCA, \
-                                            1 << MODIFIER_ORCA,
-                                            printActiveAppHandler))
-
-    printAncestryHandler = InputEventHandler(\
-        printAncestry,
-        _("Prints debug information about the ancestry of the object with focus"))
-    _keyBindings.add(keybindings.KeyBinding(
-        "F7",
-        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL),
-        1 << MODIFIER_ORCA,
-        printAncestryHandler))
-
-    printHierarchyHandler = InputEventHandler(\
-        printHierarchy,
-        _("Prints debug information about the application with focus"))
-
-    _keyBindings.add(keybindings.KeyBinding("F8", \
-                                            1 << MODIFIER_ORCA, \
-                                            1 << MODIFIER_ORCA,
-                                            printHierarchyHandler))
-
-    nextPresentationManagerHandler = InputEventHandler(\
-        _switchToNextPresentationManager,
-        _("Switches to the next presentation manager."))
-    _keyBindings.add(keybindings.KeyBinding("F10", \
-                                            1 << MODIFIER_ORCA, \
-                                            1 << MODIFIER_ORCA,
-                                            nextPresentationManagerHandler))
-
     # Create and load an app's script when it is added to the desktop
     #
     registry.registerEventListener(_onChildrenChanged,
