@@ -23,9 +23,9 @@ class.  For those wishing to override the speech generators, however,
 one can create a new instance and replace/extend the speech generators
 as they see fit."""
 
-__id__        = "$Id: speechgenerator.py,v 1.61 2006/06/10 00:54:47 wwalker Exp $"
-__version__   = "$Revision: 1.61 $"
-__date__      = "$Date: 2006/06/10 00:54:47 $"
+__id__        = "$Id: speechgenerator.py,v 1.74 2006/07/21 16:30:47 wwalker Exp $"
+__version__   = "$Revision: 1.74 $"
+__date__      = "$Date: 2006/07/21 16:30:47 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -198,23 +198,22 @@ class SpeechGenerator:
             return [label]
         else:
             return []
-        
+
     def _getSpeechForObjectName(self, obj):
         name = util.getDisplayedText(obj)
         if name:
             return [name]
         elif obj.description:
             return [obj.description]
-        else:            
+        else:
             return []
 
     def _getSpeechForObjectRole(self, obj):
-        if (settings.speechVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE)\
-           and (obj.role != rolenames.ROLE_UNKNOWN):
+        if (obj.role != rolenames.ROLE_UNKNOWN):
             return [rolenames.getSpeechForRoleName(obj)]
         else:
             return []
-        
+
     def _debugGenerator(self, generatorName, obj, already_focused, utterances):
         """Prints debug.LEVEL_FINER information regarding the speech generator.
 
@@ -262,9 +261,9 @@ class SpeechGenerator:
             utterances.extend(self._getSpeechForObjectLabel(obj))
             utterances.extend(self._getSpeechForObjectName(obj))
             utterances.extend(self._getSpeechForObjectRole(obj))
-            
+
         utterances.extend(self._getSpeechForObjectAvailability(obj))
-        
+
         self._debugGenerator("_getDefaultSpeech",
                              obj,
                              already_focused,
@@ -449,9 +448,9 @@ class SpeechGenerator:
 
         if not already_focused:
             utterances.extend(self._getSpeechForObjectRole(obj))
-            
+
         utterances.extend(self._getSpeechForObjectAvailability(obj))
-        
+
         self._debugGenerator("_getSpeechForComboBox",
                              obj,
                              already_focused,
@@ -552,7 +551,7 @@ class SpeechGenerator:
         # of treating the frame like an alert and speaking all unrelated
         # labels. We'll need to see if this has any adverse effects and
         # adjust accordingly.]]]
-        # 
+        #
         utterances = self._getDefaultSpeech(obj, already_focused)
         #utterances = self._getSpeechForAlert(obj, already_focused)
 
@@ -857,7 +856,7 @@ class SpeechGenerator:
             utterances.extend(self._getSpeechForObjectLabel(obj))
             utterances.extend(self._getSpeechForObjectName(obj))
             utterances.extend(self._getSpeechForObjectRole(obj))
-            
+
         utterances.append(percentage)
 
         self._debugGenerator("_getSpeechForProgressBar",
@@ -1044,8 +1043,20 @@ class SpeechGenerator:
         else:
             utterances = []
             utterances.extend(self._getSpeechForObjectLabel(obj))
-            # Ignore the text on the slider.
-            #utterances.extend(self._getSpeechForObjectName(obj))
+
+            # Ignore the text on the slider.  See bug 340559
+            # (http://bugzilla.gnome.org/show_bug.cgi?id=340559): the
+            # implementors of the slider support decided to put in a
+            # Unicode left-to-right character as part of the text,
+            # even though that is not painted on the screen.
+            #
+            # In Java, however, there are sliders without a label. In
+            # this case, we'll add to presentation the slider name if
+            # it exists and we haven't found anything yet.
+            #
+            if not utterances:
+                utterances.extend(self._getSpeechForObjectName(obj))
+
             utterances.extend(self._getSpeechForObjectRole(obj))
             utterances.append(valueString)
             utterances.extend(self._getSpeechForObjectAvailability(obj))
@@ -1152,11 +1163,29 @@ class SpeechGenerator:
             if settings.readTableCellRow:
                 parent = obj.parent
                 row = parent.table.getRowAtIndex(obj.index)
-                for i in range(0, parent.table.nColumns):
-                    accRow = parent.table.getAccessibleAt(row, i)
-                    cell = atspi.Accessible.makeAccessible(accRow)
+                column = parent.table.getColumnAtIndex(obj.index)
+
+                # This is an indication of whether we should speak all the
+                # table cells (the user has moved focus up or down a row),
+                # or just the current one (focus has moved left or right in
+                # the same row).
+                #
+                speakAll = True
+                if parent.__dict__.has_key("lastRow") and \
+                    parent.__dict__.has_key("lastColumn"):
+                    speakAll = (parent.lastRow != row) or \
+                           ((row == 0 or row == parent.table.nRows-1) and \
+                            parent.lastColumn == column)
+
+                if speakAll == True:
+                    for i in range(0, parent.table.nColumns):
+                        accRow = parent.table.getAccessibleAt(row, i)
+                        cell = atspi.Accessible.makeAccessible(accRow)
+                        utterances.append(util.getDisplayedText(\
+                            util.getRealActiveDescendant(cell)))
+                else:
                     utterances.append(util.getDisplayedText(\
-                        util.getRealActiveDescendant(cell)))
+                    util.getRealActiveDescendant(obj)))
             else:
                 utterances.append(util.getDisplayedText(\
                     util.getRealActiveDescendant(obj)))
@@ -1391,7 +1420,7 @@ class SpeechGenerator:
 
         Returns a list of utterances to be spoken.
         """
-        
+
         if self.speechGenerators.has_key(obj.role):
             generator = self.speechGenerators[obj.role]
         else:
@@ -1463,7 +1492,20 @@ class SpeechGenerator:
                     utterances.append(text)
                 if label and len(label):
                     utterances.append(label)
-                    
+
+            # [[[TODO: HACK - we've discovered oddness in hierarchies
+            # such as the gedit Edit->Preferences dialog.  In this
+            # dialog, we have labeled groupings of objects.  The
+            # grouping is done via a FILLER with two children - one
+            # child is the overall label, and the other is the
+            # container for the grouped objects.  When we detect this,
+            # we add the label to the overall context.]]]
+            #
+            if parent.role == rolenames.ROLE_FILLER:
+                label = util.getDisplayedLabel(parent)
+                if label and len (label):
+                    utterances.append(label)
+
             parent = parent.parent
 
         utterances.reverse()
